"use strict";
const pauseStream = require('pause-stream');
const getAuthToken = require('registry-auth-token');
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = (client) => {
    const cache = {};
    function get(url, options) {
        const key = JSON.stringify([url, options]);
        if (!cache[key]) {
            cache[key] = new Promise((resolve, reject) => {
                client.get(url, extend(url, options), (err, data, raw, res) => {
                    if (err)
                        return reject(err);
                    resolve(res);
                });
            });
        }
        return cache[key];
    }
    function getJSON(url, options) {
        const key = JSON.stringify([url, options]);
        if (!cache[key]) {
            cache[key] = new Promise((resolve, reject) => {
                client.get(url, extend(url, options), (err, data, raw, res) => {
                    if (err)
                        return reject(err);
                    resolve(data);
                });
            });
        }
        return cache[key];
    }
    const getStream = function (url, options) {
        return new Promise((resolve, reject) => {
            client.fetch(url, extend(url, options), (err, res) => {
                if (err)
                    return reject(err);
                const ps = pauseStream();
                res.pipe(ps.pause());
                resolve(ps);
            });
        });
    };
    /**
     * Extends request options with authorization headers
     */
    function extend(url, options) {
        options = options || {};
        const authToken = getAuthToken(url, { recursive: true });
        if (authToken) {
            options.headers = Object.assign({}, options.headers, {
                authorization: `${authToken.type} ${authToken.token}`
            });
        }
        return options;
    }
    return {
        get: get,
        getJSON: getJSON,
        getStream: getStream
    };
};
//# sourceMappingURL=got.js.map